﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Web.Mvc;
using System.Xml.Linq;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.MvcEdition;
using ThinkGeo.MapSuite.USDemographicMap;

namespace MapSuiteUsDemographicMap.Controllers
{
    public class DefaultController : Controller
    {
        public ActionResult Index()
        {
            Map map = new Map("Map1",
                    new System.Web.UI.WebControls.Unit(100, System.Web.UI.WebControls.UnitType.Percentage),
                    new System.Web.UI.WebControls.Unit(100, System.Web.UI.WebControls.UnitType.Percentage));
            map.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#E5E3DF"));
            map.MapUnit = GeographyUnit.DecimalDegree;
            map.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#E5E3DF"));
            map.CurrentExtent = new RectangleShape(-128.17864375, 56.9286546875, -69.11614375, 20.1903734375);

            // base map layers
            WorldMapKitWmsWebOverlay worldMapKitOverlay = new WorldMapKitWmsWebOverlay();
            map.CustomOverlays.Add(worldMapKitOverlay);

            // us states layer
            MsSql2008FeatureLayer statesLayer = new MsSql2008FeatureLayer(ConfigurationManager.ConnectionStrings["DefaultConnection"].ConnectionString, "usStatesCensus2010", "ID");
            ThematicDemographicStyleBuilder selectedStyle = new ThematicDemographicStyleBuilder(new Collection<string>() { "Population" });
            statesLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(selectedStyle.GetStyle(statesLayer.FeatureSource));
            statesLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            map.DynamicOverlay.Layers.Add("usStatesLayer", statesLayer);

            InMemoryFeatureLayer highlightLayer = new InMemoryFeatureLayer();
            highlightLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(100, 60, 180, 60), GeoColor.GeographicColors.DeepOcean);
            highlightLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay dynamicOverlay = new LayerOverlay("HightLightDynamicOverlay");
            dynamicOverlay.Layers.Add("HighLightLayer", highlightLayer);
            dynamicOverlay.IsBaseOverlay = false;

            map.CustomOverlays.Add(dynamicOverlay);

            return View(map);

        }

        [MapActionFilter]
        public string FindFeatureClick(Map map, GeoCollection<object> args)
        {
           PointShape position = new PointShape(double.Parse(args[0].ToString()), double.Parse(args[1].ToString()));

            LayerOverlay dynamicOverlay = (LayerOverlay)map.CustomOverlays["HightLightDynamicOverlay"];
            MsSql2008FeatureLayer statesLayer = (MsSql2008FeatureLayer)map.DynamicOverlay.Layers["usStatesLayer"];
            InMemoryFeatureLayer highLightLayer = (InMemoryFeatureLayer)(dynamicOverlay.Layers["HighLightLayer"]);

            highLightLayer.InternalFeatures.Clear();

            statesLayer.Open();
            Collection<Feature> selectedFeatures = statesLayer.QueryTools.GetFeaturesContaining(position, new string[] { "NAME", "Population" });
            statesLayer.Close();

            foreach (Feature feature in selectedFeatures)
            {
                highLightLayer.InternalFeatures.Add(feature.Id, feature);
            }

            return GetPopupContent(selectedFeatures);
        }

        private static string GetPopupContent(Collection<Feature> features)
        {
            string content;
            if (features.Count > 0)
            {
                StringBuilder message = new StringBuilder();
                message.AppendFormat("Country Name : {0}", features[0].ColumnValues["NAME"].Trim());
                message.AppendFormat("Country Population : {0}", features[0].ColumnValues["Population"].Trim());
                content = message.ToString();
            }
            else
            {
                content = @"Please click on a country to show its information.";
            }
            return content;
        }

    }
}
